@@ -9,7 +9,7 @@ use DBI;
 use Class::ReturnValue;
 use Encode qw();
 
-use vars qw(@ISA %DBIHandle $PrevHandle $DEBUG %TRANSDEPTH);
+use vars qw(@ISA %DBIHandle $PrevHandle $DEBUG %TRANSDEPTH %FIELDS_IN_TABLE);
 
 
 =head1 NAME
@@ -132,9 +132,10 @@ sub _UpgradeHandle {
     
     my $driver = shift;
     my $class = 'DBIx::SearchBuilder::Handle::' . $driver;
+    local $@;
     eval "require $class";
     return if $@;
-    
+
     bless $self, $class;
     return 1;
 }
@@ -331,7 +332,10 @@ sub dbh {
   my $self=shift;
   
   #If we are setting the database handle, set it.
-  $DBIHandle{$self} = $PrevHandle = shift if (@_);
+  if ( @_ ) {
+      $DBIHandle{$self} = $PrevHandle = shift;
+      %FIELDS_IN_TABLE = ();
+  }
 
   return($DBIHandle{$self} ||= $PrevHandle);
 }
@@ -674,7 +678,8 @@ sub _MakeClauseCaseInsensitive {
     my $operator = shift;
     my $value = shift;
 
-    if ($value !~ /^\d+$/) { # don't downcase integer values
+    # don't downcase integer values and things that looks like dates
+    if ($value !~ /^['"]?[-\d: ]+['"]$/) {
         $field = "lower($field)";
         $value = lc($value);
     }
@@ -755,8 +760,15 @@ sub EndTransaction {
 
     $self->TransactionDepth( $depth );
     return 1 if $depth;
-    return $self->dbh->rollback unless $action eq 'commit';
-    return $self->dbh->commit;
+
+    if ($action eq 'commit') {
+        return $self->dbh->commit;
+    }
+    else {
+        DBIx::SearchBuilder::Record::Cachable->FlushCache
+            if DBIx::SearchBuilder::Record::Cachable->can('FlushCache');
+        return $self->dbh->rollback;
+    }
 }
 
 =head3 Commit [FORCE]
@@ -878,6 +890,7 @@ sub Join {
         ALIAS1        => 'main',
         FIELD1        => undef,
         TABLE2        => undef,
+        COLLECTION2   => undef,
         FIELD2        => undef,
         ALIAS2        => undef,
         EXPRESSION    => undef,
@@ -936,11 +949,37 @@ sub Join {
             return ( $self->_NormalJoin(%args) );
         }
         $args{'SearchBuilder'}->{'aliases'} = \@new_aliases;
-    }
+    } elsif ( $args{'COLLECTION2'} ) {
+        # We're joining to a pre-limited collection.  We need to take
+        # all clauses in the other collection, munge 'main.' to a new
+        # alias, apply them locally, then proceed as usual.
+        my $collection = delete $args{'COLLECTION2'};
+        $alias = $args{ALIAS2} = $args{'SearchBuilder'}->_GetAlias( $collection->Table );
+        $args{TABLE2} = $collection->Table;
 
-    else {
+        eval {$collection->_ProcessRestrictions}; # RT hate
+
+        # Move over unused aliases
+        push @{$args{SearchBuilder}{aliases}}, @{$collection->{aliases}};
+
+        # Move over joins, as well
+        for my $join (keys %{$collection->{left_joins}}) {
+            my %alias = %{$collection->{left_joins}{$join}};
+            $alias{depends_on} = $alias if $alias{depends_on} eq "main";
+            $alias{criteria} = $self->_RenameRestriction(
+                RESTRICTIONS => $alias{criteria},
+                NEW          => $alias
+            );
+            $args{SearchBuilder}{left_joins}{$join} = \%alias;
+        }
+
+        my $restrictions = $self->_RenameRestriction(
+            RESTRICTIONS => $collection->{restrictions},
+            NEW          => $alias
+        );
+        $args{SearchBuilder}{restrictions}{$_} = $restrictions->{$_} for keys %{$restrictions};
+    } else {
         $alias = $args{'SearchBuilder'}->_GetAlias( $args{'TABLE2'} );
-
     }
 
     my $meta = $args{'SearchBuilder'}->{'left_joins'}{"$alias"} ||= {};
@@ -961,6 +1000,35 @@ sub Join {
     return ($alias);
 }
 
+sub _RenameRestriction {
+    my $self = shift;
+    my %args = (
+        RESTRICTIONS => undef,
+        OLD          => "main",
+        NEW          => undef,
+        @_,
+    );
+
+    my %return;
+    for my $key ( keys %{$args{RESTRICTIONS}} ) {
+        my $newkey = $key;
+        $newkey =~ s/^\Q$args{OLD}\E\./$args{NEW}./;
+        my @parts;
+        for my $part ( @{ $args{RESTRICTIONS}{$key} } ) {
+            if ( ref $part ) {
+                my %part = %{$part};
+                $part{field} =~ s/^\Q$args{OLD}\E\./$args{NEW}./;
+                $part{value} =~ s/^\Q$args{OLD}\E\./$args{NEW}./;
+                push @parts, \%part;
+            } else {
+                push @parts, $part;
+            }
+        }
+        $return{$newkey} = \@parts;
+    }
+    return \%return;
+}
+
 sub _NormalJoin {
 
     my $self = shift;
@@ -1012,7 +1080,7 @@ sub _BuildJoins {
 
     $self->OptimizeJoins( SearchBuilder => $sb );
 
-    my $join_clause = join ", ", ($sb->Table ." main"), @{ $sb->{'aliases'} };
+    my $join_clause = join " CROSS JOIN ", ($sb->Table ." main"), @{ $sb->{'aliases'} };
     my %processed = map { /^\S+\s+(\S+)$/; $1 => 1 } @{ $sb->{'aliases'} };
     $processed{'main'} = 1;
 
@@ -1129,10 +1197,15 @@ sub MayBeNull {
     foreach ( splice @conditions ) {
         unless ( ref $_ ) {
             push @conditions, $_;
-        } elsif ( $_->{'field'} =~ /^\Q$args{'ALIAS'}./ ) {
+        } elsif ( rindex( $_->{'field'}, "$args{'ALIAS'}.", 0 ) == 0 ) {
+            # field is alias.xxx op ... and only IS op allows NULLs
             push @conditions, lc $_->{op} eq 'is';
-        } elsif ( $_->{'value'} && $_->{'value'} =~ /^\Q$args{'ALIAS'}./ ) {
+        } elsif ( $_->{'value'} && rindex( $_->{'value'}, "$args{'ALIAS'}.", 0 ) == 0 ) {
+            # value is alias.xxx so it can not be IS op
             push @conditions, 0;
+        } elsif ( $_->{'field'} =~ /^(?i:lower)\(\s*\Q$args{'ALIAS'}\./ ) {
+            # handle 'LOWER(alias.xxx) OP VALUE' we use for case insensetive
+            push @conditions, lc $_->{op} eq 'is';
         } else {
             push @conditions, 1;
         }
@@ -1166,7 +1239,7 @@ sub MayBeNull {
                 next;
             }
 
-            my $aggreg = shift @conditions;
+            my $aggreg = uc shift @conditions;
             if ( $aggreg eq 'OR' ) {
                 # 0 OR x == x
                 next;
@@ -1183,7 +1256,7 @@ sub MayBeNull {
                 next;
             }
 
-            my $aggreg = shift @conditions;
+            my $aggreg = uc shift @conditions;
             if ( $aggreg eq 'OR' ) {
                 # 1 OR x == 1
                 my $close_p = $closing_paren->(0);
@@ -1247,6 +1320,22 @@ sub DistinctCount {
 
 }
 
+sub Fields {
+    my $self  = shift;
+    my $table = shift;
+
+    unless ( keys %FIELDS_IN_TABLE ) {
+        my $sth = $self->dbh->column_info( undef, '', '%', '%' )
+            or return ();
+        my $info = $sth->fetchall_arrayref({});
+        foreach my $e ( @$info ) {
+            push @{ $FIELDS_IN_TABLE{ lc $e->{'TABLE_NAME'} } ||= [] }, lc $e->{'COLUMN_NAME'};
+        }
+    }
+
+    return @{ $FIELDS_IN_TABLE{ lc $table } || [] };
+}
+
 
 =head2 Log MESSAGE
 
