@@ -7,7 +7,7 @@ use warnings;
 
 use base qw/DBIx::SearchBuilder::Handle/;
 
-use DBD::Oracle qw(:ora_types);
+use DBD::Oracle qw(:ora_types ORA_OCI);
          
 =head1 NAME
 
@@ -245,19 +245,28 @@ sub DistinctQuery {
     my $self = shift;
     my $statementref = shift;
     my $sb = shift;
+
     my $table = $sb->Table;
 
-    # Wrapp select query in a subselect as Oracle doesn't allow
-    # DISTINCT against CLOB/BLOB column types.
-    # Joop van de Wege: Thats correct but ORDER_BY column not in main doesn't need GROUP_BY
-    # so drop the group_by lines and add the order_by part but without the min()
     if ($sb->_OrderClause =~ /(?<!main)\./) {
+        # If we are ordering by something not in 'main', we need to GROUP
+        # BY and adjust the ORDER_BY accordingly
+        local $sb->{group_by} = [@{$sb->{group_by} || []}, {FIELD => 'id'}];
+        local $sb->{'order_by'} = [
+            map {
+                ($_->{'ALIAS'}||'') ne "main"
+                ? { %{$_}, FIELD => ((($_->{'ORDER'}||'') =~ /^des/i)?'MAX':'MIN') ."(".$_->{FIELD}.")" }
+                : $_
+            }
+            @{$sb->{'order_by'}}
+        ];
+        my $group = $sb->_GroupClause;
         my $order = $sb->_OrderClause;
-        $$statementref = "SELECT main.* FROM ( SELECT main.id FROM $$statementref $order ) distinctquery,"
-            ." $table main WHERE (main.id = distinctquery.id)";
+        $$statementref = "SELECT main.* FROM ( SELECT main.id FROM $$statementref $group $order ) distinctquery, $table main WHERE (main.id = distinctquery.id)";
     } else {
-        $$statementref = "SELECT main.* FROM ( SELECT DISTINCT main.id FROM $$statementref ) distinctquery,"
-            ." $table main WHERE (main.id = distinctquery.id) ";
+        # Wrapp select query in a subselect as Oracle doesn't allow
+        # DISTINCT against CLOB/BLOB column types.
+        $$statementref = "SELECT main.* FROM ( SELECT DISTINCT main.id FROM $$statementref ) distinctquery, $table main WHERE (main.id = distinctquery.id) ";
         $$statementref .= $sb->_GroupClause;
         $$statementref .= $sb->_OrderClause;
     }
@@ -265,6 +274,7 @@ sub DistinctQuery {
 
 
 
+
 =head2 BinarySafeBLOBs
 
 Return undef, as Oracle doesn't support binary-safe CLOBS
@@ -277,6 +287,15 @@ sub BinarySafeBLOBs {
     return(undef);
 }
 
+=head2 DatabaseVersion
+
+Returns value of ORA_OCI constant, see L<DBI/Constants>.
+
+=cut
+
+sub DatabaseVersion {
+    return ''. ORA_OCI;
+}
 
 1;
 
